'*******************************************************************************************************************
'Derinkuyu
'-------------------------------------------------------------------------------------------------------------------
'Oscar Miralles
'Start date: 13/08/2019
'End date: 01/2020
'-------------------------------------------------------------------------------------------------------------------
'Developed in Basic
'Compiled with Boriel compiler
'*******************************************************************************************************************

'-------------------------------------------------------------------------------------------------------------------
'Includes
'-------------------------------------------------------------------------------------------------------------------
#include once "fontset.bas"
#include once "titleset.bas"
#include once "tileset.bas"
#include once "objectset.bas"
#include once "spriteheroset.bas"
#include once "maskheroset.bas"
#include once "spriteset.bas"
#include once "maskset.bas"
#include once "blocksstaticdata.bas"
#include once "tilesstaticdata.bas"
#include once "spritesstaticdata.bas"
#include once "mapdata.bas"
#include once "mapherodata.bas"
#include once "mapspritesdata.bas"
#include once "objectsstaticdata.bas"
#include once "mapobjectsdata.bas"
#include once "config.bas"
#include once "engine.bas"

'-------------------------------------------------------------------------------------------------------------------
'Main game loop
'-------------------------------------------------------------------------------------------------------------------
while TRUE

	introduction()
	initGame()

	'Game loop
	while gameover = FALSE and gameend = FALSE

		moveHero()
		moveEnemies()
		detectCollisionSprites()
		detectCollisionObjects()
		if gameover = FALSE then
			printSprites(FALSE)
		end if

	wend

	showLives()

	'Game over
	if gameover = TRUE then
		gameOver()
	end if

	'Game end - Final congratulations
	if gameend = TRUE then
		endGame()
	end if

	'Pause does not work if you clicked a key during the game, they are still in memory
	'This is dirty, but it works to empty the keystrokes
	while inkey$ <> ""
	wend
	dim i as uInteger
	for i=0 to 65000
	next i
	for i=0 to 65000
	next i
	pause 0

wend

'-------------------------------------------------------------------------------------------------------------------
'Introduction screen
'-------------------------------------------------------------------------------------------------------------------
sub introduction()
	
	'Clear screen
	border COLORBLACK: paper COLORBLACK: ink COLORWHITE: cls

	'Set charset to font
	Poke uInteger 23606, @fontset(0) - 256

	'Texts are not readable because the fontset has not all the letters of alphabet, only those used in the game, to save memory
	center(12, ":34B. =", COLORBLUE) 'LEFT: O
	center(13, "@756B. >", COLORBLUE) 'RIGHT: P
	center(14, "8C;>. A>/13", COLORBLUE) 'JUMP: SPACE'

	center(16, ">@3AA / 93G B= AB/@B", COLORWHITE) 'PRESS A KEY TO START

	center(20, "I =A1/@ ;7@/::3A &$%-", COLORMAGENTA) '@ Oscar Miralles 2019

	introTitle()
	introAnimation()

	'Clear screen
	pause 0: cls

end sub

'-------------------------------------------------------------------------------------------------------------------
'Game over
'-------------------------------------------------------------------------------------------------------------------
sub gameOver()

	'Set charset to font
	Poke uInteger 23606, @fontset(0) - 256
	
	center((MAPSCREENHEIGHT*TILESIZE)/2, "5/;3 =D3@", COLORWHITE) 'GAME OVER

end sub

'-------------------------------------------------------------------------------------------------------------------
'Game end - Final congratulations
'-------------------------------------------------------------------------------------------------------------------
sub endGame()

	'Set charset to font
	Poke uInteger 23606, @fontset(0) - 256
	
	center(2, "                                 ", COLORBLACK) 'Empty background
	center(3, "       1=<5@/BC:/B7=<A !!!       ", COLORMAGENTA) 'CONGRATULATIONS !!!
	center(4, "                                 ", COLORBLACK) 'Empty background
	center(5, "                                 ", COLORBLACK) 'Empty background
	center(6, "   G=C 6/D3 A/D32 B63 >@7<13AA   ", COLORCYAN) 'You have saved the princess
	center(7, "                                 ", COLORBLACK) 'Empty background
	center(8, "         <=E G=C 1/< :7D3        ", COLORCYAN) 'now you can live
	center(9, "                                 ", COLORBLACK) 'Empty background
	center(10, "          6/>>G B=53B63@         ", COLORCYAN) 'happy together
	center(11, "                                 ", COLORBLACK) 'Empty background

	'Pause does not work if you clicked a key during the game, they are still in memory
	'This is dirty, but it works to empty the keystrokes
	dim i as uInteger
	for i = 0 to 65000
	next i
	for i = 0 to 65000
	next i
	for i = 0 to 65000
	next i
	for i = 0 to 65000
	next i

	gameend = TRUE

	'Clear screen
	border COLORBLACK: paper COLORBLACK: ink COLORWHITE: cls

end sub

'-------------------------------------------------------------------------------------------------------------------
'Intro screen - Title
'-------------------------------------------------------------------------------------------------------------------
sub introTitle()

	dim i, j, k as uByte

	'Set charset to title
	Poke uInteger 23606, @titleset(0) - 256
	
	k = 0
	for i = 0 to 1
		for j = 0 to 10
			print paper COLORBLACK; ink COLORYELLOW; bright TRUE; at 1 + i + SCREENOFFSETY, (MAPSCREENWIDTH*TILESIZE - 11)/2 + j + SCREENOFFSETX; chr(32 + k)
			k = k + 1
		next j
	next i

end sub

'-------------------------------------------------------------------------------------------------------------------
'Intro screen - Animation of hero and princess
'-------------------------------------------------------------------------------------------------------------------
sub introAnimation()

	dim i, j, k, l as uByte
	dim frame1, frameused, chr1 as uByte

	over 0

	'--------------------------------------
	'Princess
	'--------------------------------------
	'Set charset to objectset
	Poke uInteger 23606, @objectset(0) - 256
	'Chr princess
	chr1 = objectsstaticdata(OBJECTFINAL, OBJECTSTATICCHR)

	k = 0
	'Depends on size y
	for i = 0 to objectsstaticdata(OBJECTFINAL, OBJECTSTATICSIZEY) - 1

		'As many chr as size x
		for j = 0 to objectsstaticdata(OBJECTFINAL, OBJECTSTATICSIZEX) - 1

			print paper COLORBLACK; ink COLORCYAN; bright TRUE; at 6 + i + SCREENOFFSETY, 12 + j + SCREENOFFSETX; chr(chr1 + k)

			k = k + 1
		next j
	next i

	'--------------------------------------
	'Floor
	'--------------------------------------
	'Set charset to tileset
	Poke uInteger 23606, @tileset(0) - 256
	'Chr grass
	chr1 = blocksstaticdata(1, BLOCKSTATICCHR1)

	for i = 3 to 23
		print paper COLORBLACK; ink COLORGREEN; bright TRUE; at 9 + SCREENOFFSETY, i + SCREENOFFSETX; chr(chr1)
	next i

	'--------------------------------------
	'Hero
	'--------------------------------------
	'Set charset to spriteheroset
	Poke uInteger 23606, @spriteheroset(0) - 256

	frame1 = 1

	for l = 2 to 10
		
		'Chr
		'Frame 0 is repeated to save 1 sprite from spriteset and make walk nicer
		'Frames: 0 - 1 - 0 - 2
		frameused = frame1
		If frame1 = 2 then 
			frameused = 0
		else if frame1 = 3 then
			frameused = 2
		end if

		'Hero walking to the right
		chr1 = frameused * (spritesstaticdata(HERO, SPRITESTATICSIZEX) * spritesstaticdata(HERO, SPRITESTATICSIZEY))
		chr1 = spritesstaticdata(HERO, SPRITESTATICCHRRIGHT) + chr1

		k = 0
		'Depends on size y
		for i = 0 to spritesstaticdata(HERO, SPRITESTATICSIZEY) - 1

			'Space to remove
			print paper COLORBLACK; ink COLORBLACK; bright TRUE; at 6 + i + SCREENOFFSETY, l + SCREENOFFSETX; chr(CHRSPACE)

			'As many chr as size x
			for j = 0 to spritesstaticdata(HERO, SPRITESTATICSIZEX) - 1

				print paper COLORBLACK; ink COLORWHITE; bright TRUE; at 6 + i + SCREENOFFSETY, 1 + j + l + SCREENOFFSETX; chr(chr1 + k)

				k = k + 1
			next j
		next i

		'Last frame
		frame1 = frame1 + 1
		if frame1 = spritesstaticdata(HERO, SPRITESTATICFRAMES) then
			frame1 = 0
		end if

        beep .001, 1

		pause 10

	next l

end Sub

