'*******************************************************************************************************************
'Engine
'-------------------------------------------------------------------------------------------------------------------
'Functions that form the core of the game: printing sprites, screen, collisions, etc. 
'*******************************************************************************************************************

'Current and Old sprite dynamic data
Dim sprites(NUMSPRITES-1, SPRITESTRUCT-1) as uByte

'Current objects dynamic data
Dim objects(NUMOBJECTS-1, OBJECTSTRUCT-1) as uByte

'Map
Dim map(MAPSCREENHEIGHT-1, MAPSCREENWIDTH-1, MAPSTRUCT-1) as uByte

'Initial coordinates at entering a screen
dim initY as uByte
dim initX as uByte
dim initDirY as uByte
dim initDirX as uByte
dim initframe as uByte
dim initFrameJump as uByte
dim initStatus as uByte

'Map
dim currentscreenx as ubyte
dim currentscreeny as ubyte
dim previousscreenx as ubyte
dim previousscreeny as ubyte

'Lives
dim lives as ubyte
dim gameover as ubyte
dim gameend as ubyte

'Objects
dim objectsCollected as ubyte

'Jump
dim jumpStep as ubyte
dim jumpDirection as ubyte
dim jumpStepsY(JUMPSTEPS-1) as byte => {-1, -1, -1, -1, 0, 1, 1, 1, 1} 'Increments of y
dim jumpStepsX(JUMPSTEPS-1) as ubyte => {0, 1, 1, 1, 1, 1, 1, 1, 0} 'Increments of x

'Fall
dim fallCount as ubyte

'-------------------------------------------------------------------------------------------------------------------
'Get the current first character from the spriteset to print.
'Depends on the frame, direction and size of sprite
'-------------------------------------------------------------------------------------------------------------------
Function getChrSprite (spritenum as uByte) as ubyte
	
	dim chr1, chrinc, frameused, size as ubyte

	size = (spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICSIZEX) * spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICSIZEY))

	'Hero jumping or falling
	if sprites(spritenum, SPRITESTATUS) = SPRITEJUMPING or sprites(spritenum, SPRITESTATUS) = SPRITEFALLING then
		'Take horizontal chr
		if sprites(spritenum, SPRITEDIRX) <> DIRNONE then
			
			'Chr increment
			chrinc = sprites(spritenum, SPRITEFRAMEJUMP) * size

			'Right
			if sprites(spritenum, SPRITEDIRX) = DIRRIGHT then
				'First chr
				chr1 = spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICCHRJUMPRIGHT) + chrinc
			'Left
			elseif sprites(spritenum, SPRITEDIRX) = DIRLEFT then
				'First chr
				chr1 = spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICCHRJUMPLEFT) + chrinc
			end if

		end if

	'Hero dying
	else if sprites(spritenum, SPRITESTATUS) = SPRITEDYING then

		'Chr increment
		chrinc = sprites(spritenum, SPRITEFRAMEDIE) * size

		'First chr
		chr1 = spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICCHRDIE) + chrinc

	'Other (Any sprite)
	else

		'Horizontal
		if sprites(spritenum, SPRITEDIRX) <> DIRNONE then
			
			'Current frame
			frameused = sprites(spritenum, SPRITEFRAME)

			'Hero
			'Frame 0 is repeated to save 1 sprite from spriteset and make walk nicer
			'Frames: 0 - 1 - 0 - 2
			if spritenum = HERO then
				If sprites(spritenum, SPRITEFRAME) = 2 then 
					frameused = 0
				else if sprites(spritenum, SPRITEFRAME) = 3 then
					frameused = 2
				end if
			end if

			'Chr increment
			chrinc = frameused * size

			'Right
			if sprites(spritenum, SPRITEDIRX) = DIRRIGHT then
				'First chr
				chr1 = spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICCHRRIGHT) + chrinc
			'Left
			elseif sprites(spritenum, SPRITEDIRX) = DIRLEFT then
				'First chr
				chr1 = spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICCHRLEFT) + chrinc
			end if

		end if

		'Vertical (Only for stalactit/fish)
		if sprites(spritenum, SPRITEDIRY) <> DIRNONE then
			
			'Chr increment
			chrinc = sprites(spritenum, SPRITEFRAME) * size

			'Vertical = Right/Left - We don't have special chr for vertical anymore
			'Right = Up
			'Left = Down

			'Up
			if sprites(spritenum, SPRITEDIRY) = DIRUP then
				'First chr
				chr1 = spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICCHRRIGHT) + chrinc
			'Down
			elseif sprites(spritenum, SPRITEDIRY) = DIRDOWN then
				'First chr
				chr1 = spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICCHRLEFT) + chrinc
			end if

		end if

	end if

	return chr1

End Function

'-------------------------------------------------------------------------------------------------------------------
'Get the current character on the map at position Y,X.
'Find the tile and then block (depending on Y) and then the character (depending on X) 
'-------------------------------------------------------------------------------------------------------------------
function getTileChrAt(y as ubyte, x as ubyte) as ubyte

	dim chr1, tile, posy, posx as uByte

	'Tile at Y,X
	tile = map(y/TILESIZE, x/TILESIZE, MAPTILE)

	'Mod gives you the reminder
	posy = y mod TILESIZE
	posx = x mod TILESIZE

	'The reminder tells us in which position of the tile we are
	' X: 0 1 2
	'Y: -------
	'0:  X X X
	'1:  X X X
	'2:  X X X

	'Y0 = block1 - X0 = chr1
	if posy = 0 and posx = 0 then
		chr1 = blocksstaticdata(tilesstaticdata(tile, TILESTATICBLOCK1), BLOCKSTATICCHR1)
	'Y0 = block1 - X1 = chr2
	elseif posy = 0 and posx = 1 then
		chr1 = blocksstaticdata(tilesstaticdata(tile, TILESTATICBLOCK1), BLOCKSTATICCHR2)
	'Y0 = block1 - X2 = chr3
	elseif posy = 0 and posx = 2 then
		chr1 = blocksstaticdata(tilesstaticdata(tile, TILESTATICBLOCK1), BLOCKSTATICCHR3)
	'Y1 = block2 - X0 = chr1
	elseif posy = 1 and posx = 0 then
		chr1 = blocksstaticdata(tilesstaticdata(tile, TILESTATICBLOCK2), BLOCKSTATICCHR1)
	'Y1 = block2 - X1 = chr2
	elseif posy = 1 and posx = 1 then
		chr1 = blocksstaticdata(tilesstaticdata(tile, TILESTATICBLOCK2), BLOCKSTATICCHR2)
	'Y1 = block2 - X2 = chr3
	elseif posy = 1 and posx = 2 then
		chr1 = blocksstaticdata(tilesstaticdata(tile, TILESTATICBLOCK2), BLOCKSTATICCHR3)
	'Y2 = block3 - X0 = chr1
	elseif posy = 2 and posx = 0 then
		chr1 = blocksstaticdata(tilesstaticdata(tile, TILESTATICBLOCK3), BLOCKSTATICCHR1)
	'Y2 = block3 - X1 = chr2
	elseif posy = 2 and posx = 1 then
		chr1 = blocksstaticdata(tilesstaticdata(tile, TILESTATICBLOCK3), BLOCKSTATICCHR2)
	'Y2 = block3 - X2 = chr3
	else
		chr1 = blocksstaticdata(tilesstaticdata(tile, TILESTATICBLOCK3), BLOCKSTATICCHR3)
	end if

	'Chr at the position
	return chr1

end function

'-------------------------------------------------------------------------------------------------------------------
'Get the current tile type on the map at position Y,X.
'Find the tile and then block (depending on Y) and then the type is defined in block
'-------------------------------------------------------------------------------------------------------------------
function getTileTypeAt(y as ubyte, x as ubyte) as ubyte

	dim type, tile, posy as uByte

	'Tile at Y,X
	tile = map(y/TILESIZE, x/TILESIZE, MAPTILE)

	'Mod gives you the reminder
	posy = y mod TILESIZE

	'The reminder tells us in which position of the tile we are
	' X: 0 1 2
	'Y: -------
	'0:  X X X
	'1:  X X X
	'2:  X X X

	'We only need the Y, because we have only 3 types, one for each line (= block)
	'Y0 = block1
	if posy = 0 then
		type = tilesstaticdata(tile, TILESTATICTYPE1)
	'Y1 = block2
	elseif posy = 1 then
		type = tilesstaticdata(tile, TILESTATICTYPE2)
	'Y2 = block3
	else
		type = tilesstaticdata(tile, TILESTATICTYPE3)
	end if

	'Type at the position
	return type

end function

'-------------------------------------------------------------------------------------------------------------------
'Get the current character color on the map at position Y,X.
'Find the tile and then block (depending on Y) and then the color is defined in block
'-------------------------------------------------------------------------------------------------------------------
function getTileColorAt(y as ubyte, x as ubyte) as ubyte

	dim color, tile, posy, posx as uByte

	'Tile at Y,X
	tile = map(y/TILESIZE, x/TILESIZE, MAPTILE)

	'Mod gives you the reminder
	posy = y mod TILESIZE

	'The reminder tells us in which position of the tile we are
	' X: 0 1 2
	'Y: -------
	'0:  X X X
	'1:  X X X
	'2:  X X X

	'We only need the Y, because we have only 3 colors, one for each line (= block)
	'Y0 = block1
	if posy = 0 then
		color = blocksstaticdata(tilesstaticdata(tile, TILESTATICBLOCK1), BLOCKSTATICCOLOR)
	'Y1 = block2
	elseif posy = 1 then
		color = blocksstaticdata(tilesstaticdata(tile, TILESTATICBLOCK2), BLOCKSTATICCOLOR)
	'Y2 = block3
	else
		color = blocksstaticdata(tilesstaticdata(tile, TILESTATICBLOCK3), BLOCKSTATICCOLOR)
	end if

	'Color at the position
	return color

end function

'-------------------------------------------------------------------------------------------------------------------
'Get if the current character is solid or not on the map at position Y,X.
'Find the tile type and see if it is solid
'-------------------------------------------------------------------------------------------------------------------
function isSolidFloor(y as uByte, x as uByte) as uByte

	dim solid as uByte = FALSE

	'If we are in the screen
    if y <= (MAPSCREENHEIGHT * TILESIZE) and x <= (MAPSCREENWIDTH * TILESIZE) then
    	'If tile is solid or tresspassable then return solid
		if getTileTypeAt(y, x) = TILESOLID or getTileTypeAt(y, x) = TILESOLIDTRESPASS then
			solid = TRUE
		end if
	end if

	'Solid True/False
	return solid

end function

'-------------------------------------------------------------------------------------------------------------------
'Get the PAPER value from a color.
'color (INK + 8 * PAPER + 64 * BRIGHT)
'-------------------------------------------------------------------------------------------------------------------
function getPaper(color as uByte) as uByte
	
	'00111000
	return (color bAnd 56)/8

End Function

'-------------------------------------------------------------------------------------------------------------------
'Get the INK value from a color.
'color (INK + 8 * PAPER + 64 * BRIGHT)
'-------------------------------------------------------------------------------------------------------------------
function getInk(color as uByte) as uByte
	
	'00000111
	return (color bAnd 7)

End Function

'-------------------------------------------------------------------------------------------------------------------
'Get the BRIGHT value from a color.
'color (INK + 8 * PAPER + 64 * BRIGHT)
'-------------------------------------------------------------------------------------------------------------------
function getBright(color as uByte) as uByte
	
	'01000000
	return (color bAnd 64)

End Function

'-------------------------------------------------------------------------------------------------------------------
'Initialises the Game.
'Always at the begining of a new game.
'-------------------------------------------------------------------------------------------------------------------
sub initGame()

	'Init vars with very first value
	'Current and previous screen are the same
	currentscreeny = STARTSCREENY 'Start screen Y
	currentscreenx = STARTSCREENX 'Start screen X
	previousscreeny = currentscreeny
	previousscreenx = currentscreenx
	lives = MAXLIVES
	gameover = FALSE
	gameend = FALSE
	fallCount = 0

	'Init functions
	getReady()
	initObjects()
	initScreen(currentscreeny, currentscreenx, FALSE)

end sub

'-------------------------------------------------------------------------------------------------------------------
'Initialises the current screen Y,X.
'Always at the begining of a new life, after init game or after dying.
'hasDied = If hero has died, we will make him appear from the position he had at entering the screen
'-------------------------------------------------------------------------------------------------------------------
sub initScreen(screeny as ubyte, screenx as ubyte, hasDied as uByte)

	'Clear screen
	border COLORBLACK: paper COLORBLACK: ink COLORWHITE: cls

	'Set the screen Y,X
	currentscreeny = screeny
	currentscreenx = screenx

	'Init functions
	showLives()
	showObjectsCollected()
	initSprites()
	initHero(hasDied)
	initMap()
	printMap()
	printObjects()
	printSprites(FALSE)

end sub

'-------------------------------------------------------------------------------------------------------------------
'Shows lives counter.
'-------------------------------------------------------------------------------------------------------------------
sub showLives()

	'Set charset to fontset
	Poke uInteger 23606, @fontset(0) - 256

	'Lives: #
	print paper COLORBLACK; ink COLORWHITE; bright TRUE; at LIVESY + SCREENOFFSETY, LIVESX + SCREENOFFSETX; ":7D3A. "; chr(CHRNUM0 + lives) 'Lives: ' + NUMBERS 'CHR(36) is the 0

end sub

'-------------------------------------------------------------------------------------------------------------------
'Init the objects array with the initial DATA defined at mapobjectsdata.bas.
'-------------------------------------------------------------------------------------------------------------------
sub initObjects()

	dim i as ubyte
	dim objectdata as uByte

	'Copy all initial values of objects from the map
	restore Mapobjectsdata
	for i = 0 to NUMOBJECTS - 1

		'Current object
		read objectdata
		objects(i, OBJECTID) = objectdata
		read objectdata
		objects(i, OBJECTOBJID) = objectdata
		read objectdata
		objects(i, OBJECTSCREENY) = objectdata
		read objectdata
		objects(i, OBJECTSCREENX) = objectdata
		read objectdata
		objects(i, OBJECTY) = objectdata
		read objectdata
		objects(i, OBJECTX) = objectdata
		read objectdata
		objects(i, OBJECTCOLOR) = objectdata
		read objectdata
		objects(i, OBJECTLINKEDID) = objectdata
		'Always Active at the begining
		objects(i, OBJECTSTATUS) = OBJECTACTIVE

	next i

end sub

'-------------------------------------------------------------------------------------------------------------------
'Init the sprites array with the initial DATA defined.
'Hero data at mapherodata.bas.
'Enemies data at mapspritesdata.bas.
'-------------------------------------------------------------------------------------------------------------------
sub initSprites()

	dim i, j, im, jm as uByte
	dim spritedata as uByte

	'Copy all initial values of sprites of current screen

	'------------------------------------------
	'HERO
	'------------------------------------------
	'DATA has less information because many vars have always the same value, this way we save memory.
	restore Mapherodata
	for im = 0 to MAPHEIGHT - 1
		for jm = 0 to MAPWIDTH - 1

			'We read DATA in a loop but not use it until we get to current screen, then we copy it in vars

			'Current sprite
			if currentscreeny = im and currentscreenx = jm then
				sprites(HERO, SPRITEID) = HERO
				sprites(HERO, SPRITEFRAME) = SPRITEFIRSTFRAME 'Always 0
				sprites(HERO, SPRITEFRAMEJUMP) = SPRITEFIRSTFRAME 'Always 0
				sprites(HERO, SPRITEFRAMEDIE) = SPRITEFIRSTFRAME 'Always 0
				sprites(HERO, SPRITEMINY) = 0 'Not used
				sprites(HERO, SPRITEMINX) = 0 'Not used
				sprites(HERO, SPRITEMAXY) = 0 'Not used
				sprites(HERO, SPRITEMAXX) = 0 'Not used
				sprites(HERO, SPRITEDIRY) = DIRNONE
				sprites(HERO, SPRITECOLOR) = COLORWHITE + COLORBRIGHT 'color (INK + 8 * PAPER + 64 * BRIGHT)
				sprites(HERO, SPRITESTATUS) = SPRITEACTIVE 'Always Active at the begining
			end if
			read spritedata
			if currentscreeny = im and currentscreenx = jm then
				sprites(HERO, SPRITEY) = spritedata
				sprites(HERO, SPRITEINITY) = spritedata
			end if
			read spritedata
			if currentscreeny = im and currentscreenx = jm then
				sprites(HERO, SPRITEX) = spritedata
				sprites(HERO, SPRITEINITX) = spritedata
			end if
			read spritedata
			if currentscreeny = im and currentscreenx = jm then
				sprites(HERO, SPRITEDIRX) = spritedata
				sprites(HERO, SPRITEINITDIRX) = spritedata
			end if

			'Init Old sprite vars with current sprite vars
			if currentscreeny = im and currentscreenx = jm then
				setOldSprite(HERO)
			end if

			'We do read DATA only till current screen, the next screens are not interesting now, so the loop is shorter
			if currentscreeny = im and currentscreenx = jm then
				exit for
			end if

		next jm

		'We do read DATA only till current screen, the next screens are not interesting now, so the loop is shorter
		if currentscreeny = im and currentscreenx = jm then
			exit for
		end if

	next im

	'------------------------------------------
	'Enemies
	'------------------------------------------
	'DATA has more information because many vars are dynamic depending on screen.
	restore Mapspritesdata
	for im = 0 to MAPHEIGHT - 1
		for jm = 0 to MAPWIDTH - 1
			'There is only 1 enemy per screen, but I keep this loop in case more are added
			for i = 1 to NUMSPRITES - 1

				'We read DATA in a loop but not use it until we get to current screen, then we copy it in vars

				'Current sprite
				read spritedata
				if currentscreeny = im and currentscreenx = jm then
					sprites(i, SPRITEID) = spritedata
					sprites(i, SPRITEFRAME) = SPRITEFIRSTFRAME 'Always 0
					sprites(i, SPRITEFRAMEJUMP) = SPRITEFIRSTFRAME 'Always 0, not used
					sprites(i, SPRITEFRAMEDIE) = SPRITEFIRSTFRAME 'Always 0, not used
					sprites(i, SPRITESTATUS) = SPRITEACTIVE 'Always Active at the begining
				end if
				read spritedata
				if currentscreeny = im and currentscreenx = jm then
					sprites(i, SPRITEY) = spritedata
					sprites(i, SPRITEINITY) = spritedata
				end if
				read spritedata
				if currentscreeny = im and currentscreenx = jm then
					sprites(i, SPRITEX) = spritedata
					sprites(i, SPRITEINITX) = spritedata
				end if
				read spritedata
				if currentscreeny = im and currentscreenx = jm then
					sprites(i, SPRITEMINY) = spritedata
				end if
				read spritedata
				if currentscreeny = im and currentscreenx = jm then
					sprites(i, SPRITEMINX) = spritedata
				end if
				read spritedata
				if currentscreeny = im and currentscreenx = jm then
					sprites(i, SPRITEMAXY) = spritedata
				end if
				read spritedata
				if currentscreeny = im and currentscreenx = jm then
					sprites(i, SPRITEMAXX) = spritedata
				end if
				read spritedata
				if currentscreeny = im and currentscreenx = jm then
					sprites(i, SPRITEDIRY) = spritedata
				end if
				read spritedata
				if currentscreeny = im and currentscreenx = jm then
					sprites(i, SPRITEDIRX) = spritedata
					sprites(i, SPRITEINITDIRX) = spritedata
				end if
				read spritedata
				if currentscreeny = im and currentscreenx = jm then
					sprites(i, SPRITECOLOR) = spritedata
				end if

				'Init Old sprite vars with current sprite vars
				if currentscreeny = im and currentscreenx = jm then
					setOldSprite(i)
				end if

			next i

			'We do read DATA only till current screen, the next screens are not interesting now, so the loop is shorter
			if currentscreeny = im and currentscreenx = jm then
				exit for
			end if

		next jm

		'We do read DATA only till current screen, the next screens are not interesting now, so the loop is shorter
		if currentscreeny = im and currentscreenx = jm then
			exit for
		end if

	next im

end sub

'-------------------------------------------------------------------------------------------------------------------
'Init the hero vars.
'Always at the begining of a new life, after init game or after dying.
'hasDied = If hero has died, we will make him appear from the position he had at entering the screen
'-------------------------------------------------------------------------------------------------------------------
sub initHero(hasDied as uByte)

	'Set the coordinates to those of when he entered screen if coming from another screen
	if previousscreenx <> currentscreenx or previousscreeny <> currentscreeny then
		'If hero has not died, we are entering from another screen
		if hasDied = FALSE then
			sprites(HERO, SPRITEY) = initY
			sprites(HERO, SPRITEX) = initX
			sprites(HERO, SPRITEFRAME) = initframe
			sprites(HERO, SPRITEFRAMEJUMP) = initFrameJump
			sprites(HERO, SPRITEFRAMEDIE) = SPRITEFIRSTFRAME 'No changes from screen to screen, always 0
			sprites(HERO, SPRITEDIRY) = initDirY
			sprites(HERO, SPRITEDIRX) = initDirX
			sprites(HERO, SPRITESTATUS) = initStatus

		'If hero has died, leave coordinates to the ones saved when he entered this screen last time, and no jump
		else
			resetJump()
			sprites(HERO, SPRITEY) = initY
			sprites(HERO, SPRITEX) = initX
			sprites(HERO, SPRITEFRAME) = SPRITEFIRSTFRAME
			sprites(HERO, SPRITEDIRY) = initDirY
			sprites(HERO, SPRITEDIRX) = initDirX
			sprites(HERO, SPRITESTATUS) = SPRITEACTIVE
		end if
	
	'Not coming from another screen
	else
		resetJump()
		'If hero has died, leave coordinates to the ones defined for this screen
		if hasDied = TRUE then
			sprites(HERO, SPRITEY) = sprites(HERO, SPRITEINITY)
			sprites(HERO, SPRITEX) = sprites(HERO, SPRITEINITX)
			sprites(HERO, SPRITEFRAME) = SPRITEFIRSTFRAME
			sprites(HERO, SPRITEDIRY) = DIRNONE
			sprites(HERO, SPRITEDIRX) = sprites(HERO, SPRITEINITDIRX)
			sprites(HERO, SPRITESTATUS) = SPRITEACTIVE
		else
			initY = sprites(HERO, SPRITEY)
			initX = sprites(HERO, SPRITEX)
		end if
	end if

	'Set Old sprite vars
	setOldSprite(HERO)

end sub

'-------------------------------------------------------------------------------------------------------------------
'Resets the jump vars.
'Jump has finished or has been cut.
'-------------------------------------------------------------------------------------------------------------------
sub resetJump()
	jumpStep = 0
	initFrameJump = 0
	jumpDirection = DIRNONE
	sprites(HERO, SPRITEFRAMEJUMP) = SPRITEFIRSTFRAME
	sprites(HERO, SPRITESTATUS) = SPRITEACTIVE
	sprites(HERO, SPRITEDIRY) = DIRNONE
	fallCount = 0
end sub

'-------------------------------------------------------------------------------------------------------------------
'Init the map array with the initial DATA defined at mapdata.bas.
'Map array has data from current screen only
'DATA is compressed to save memory:
'A negative number (-N) means that the next tile will be repeated N times:
'-2, 3 : tile 3 will be repeated 2 times
'Data is only the tile, position Y,X is known after decompression
'-------------------------------------------------------------------------------------------------------------------
sub initMap()

	dim i, j, k, im, jm, s, c as uByte
	dim tile, tile2 as uByte

	'------------------------------------------
	'Tiles
	'------------------------------------------
	'Copy map of current screen
	restore Mapdata
	for im = 0 to MAPHEIGHT - 1
		for jm = 0 to MAPWIDTH - 1
			for i = 0 to MAPSCREENHEIGHT - 1

				j = 0
				while j <= MAPSCREENWIDTH - 1

					'We read DATA in a loop but not use it until we get to current screen, then we copy it in vars

					read tile

					'Map data is compressed.
					'< 0 - Number of tiles to add
					'tile is an uByte, so when reading a negative it stores it as a number from 256 - 128 (256=0 to 128=128)
					'Positive goes from 0 to 128 (0=0 to 128=128)
					if tile > 128 then

						'To know the real value we need to invert it (256=0 to 128=128)
						tile = 256 - tile
	
						'If < 0 then we read the next tile in order to know which tile2 must be repeated tile times
						read tile2
						for s = 1 to tile
	
							if currentscreeny = im and currentscreenx = jm then
								'Tile to be repeated (not 0 anymore, now we can repeat any tile)
								map(i, j, MAPTILE) = tile2
							end if

							j = j +  1

						next s

					'> 0 - Normal tile, we take the value of tile directly
					else

						if currentscreeny = im and currentscreenx = jm then
							map(i, j, MAPTILE) = tile
						end if

						j = j + 1

					end if

				wend

			next i

			'We do read DATA only till current screen, the next screens are not interesting now, so the loop is shorter
			if currentscreeny = im and currentscreenx = jm then
				exit for
			end if

		next jm

		'We do read DATA only till current screen, the next screens are not interesting now, so the loop is shorter
		if currentscreeny = im and currentscreenx = jm then
			exit for
		end if

	next im

	'------------------------------------------
	'Doors/Final
	'------------------------------------------
	'If there is a door or the final princess in this screen, active, set the tiles at its position to solid
	for i = 0 to NUMOBJECTS - 1
		'Object in screen
		if objects(i, OBJECTSCREENY) = currentscreeny and objects(i, OBJECTSCREENX) = currentscreenx then
			'Is door or final princess
			if objectsstaticdata(objects(i, OBJECTID), OBJECTSTATICTYPE) = OBJECTDOOR or objectsstaticdata(objects(i, OBJECTID), OBJECTSTATICTYPE) = OBJECTFINAL then
				'Is active
				if objects(i, OBJECTSTATUS) = OBJECTACTIVE then
					'The tile at the position becomes solid but transparent
					map(objects(i, OBJECTY)/TILESIZE, objects(i, OBJECTX)/TILESIZE, MAPTILE) = MAPTILEEMPTYSOLID
				end if
			end if
		end if
	next i

end sub

'-------------------------------------------------------------------------------------------------------------------
'Print the current screen of the map on screen.
'From the tile we can extract the blocks (with color) and the characters to print.
'-------------------------------------------------------------------------------------------------------------------
sub printMap()

	dim x, y, xt, yt, chr1, chr2, chr3, chr4, chr5, chr6, chr7, chr8, chr9, tilecolor1, tilecolor2, tilecolor3 as ubyte

	for y = 0 to MAPSCREENHEIGHT - 1
		for x = 0 to MAPSCREENWIDTH - 1

			'Print only if not empty
			if map(y, x, MAPTILE) <> MAPTILEEMPTY then

				'Set charset to tileset
				Poke uInteger 23606, @tileset(0) - 256

				'Get the characters to print from the tile (tile is 3 x 3)
				'  CHR: 1 2 3
				'Block: -------
				'1:     1 2 3
				'2:     4 5 6
				'3:     7 8 9

				'Chr
				'block1 - chr1 = chr1
				chr1 = blocksstaticdata(tilesstaticdata(map(y, x, MAPTILE), TILESTATICBLOCK1), BLOCKSTATICCHR1)
				'block1 - chr2 = chr2
				chr2 = blocksstaticdata(tilesstaticdata(map(y, x, MAPTILE), TILESTATICBLOCK1), BLOCKSTATICCHR2)
				'block1 - chr3 = chr3
				chr3 = blocksstaticdata(tilesstaticdata(map(y, x, MAPTILE), TILESTATICBLOCK1), BLOCKSTATICCHR3)
				'block2 - chr1 = chr4
				chr4 = blocksstaticdata(tilesstaticdata(map(y, x, MAPTILE), TILESTATICBLOCK2), BLOCKSTATICCHR1)
				'block2 - chr2 = chr5
				chr5 = blocksstaticdata(tilesstaticdata(map(y, x, MAPTILE), TILESTATICBLOCK2), BLOCKSTATICCHR2)
				'block2 - chr3 = chr6
				chr6 = blocksstaticdata(tilesstaticdata(map(y, x, MAPTILE), TILESTATICBLOCK2), BLOCKSTATICCHR3)
				'block3 - chr1 = chr7
				chr7 = blocksstaticdata(tilesstaticdata(map(y, x, MAPTILE), TILESTATICBLOCK3), BLOCKSTATICCHR1)
				'block3 - chr2 = chr8
				chr8 = blocksstaticdata(tilesstaticdata(map(y, x, MAPTILE), TILESTATICBLOCK3), BLOCKSTATICCHR2)
				'block3 - chr3 = chr9
				chr9 = blocksstaticdata(tilesstaticdata(map(y, x, MAPTILE), TILESTATICBLOCK3), BLOCKSTATICCHR3)

				'color (INK + 8 * PAPER + 64 * BRIGHT)
				tilecolor1 = blocksstaticdata(tilesstaticdata(map(y, x, MAPTILE), TILESTATICBLOCK1), BLOCKSTATICCOLOR)
				tilecolor2 = blocksstaticdata(tilesstaticdata(map(y, x, MAPTILE), TILESTATICBLOCK2), BLOCKSTATICCOLOR)
				tilecolor3 = blocksstaticdata(tilesstaticdata(map(y, x, MAPTILE), TILESTATICBLOCK3), BLOCKSTATICCOLOR)

				'Position Y,X + Offset
				xt = (x * TILESIZE) + SCREENOFFSETX
				yt = (y * TILESIZE) + SCREENOFFSETY

				'Print on screen
				print paper getPaper(tilecolor1); ink getInk(tilecolor1); bright getBright(tilecolor1); at yt, xt; chr(chr1) + chr(chr2) + chr(chr3)
				print paper getPaper(tilecolor2); ink getInk(tilecolor2); bright getBright(tilecolor2); at yt+1, xt; chr(chr4) + chr(chr5) + chr(chr6)
				print paper getPaper(tilecolor3); ink getInk(tilecolor3); bright getBright(tilecolor3); at yt+2, xt; chr(chr7) + chr(chr8) + chr(chr9)

			end if

		next x
	next y

end sub

'-------------------------------------------------------------------------------------------------------------------
'Print the objects of the current screen of the map on screen.
'-------------------------------------------------------------------------------------------------------------------
sub printObjects()

	dim x, y, chr1, objcolor, i, j, k, l as ubyte

	'Loop all objects
	for i = 0 to NUMOBJECTS - 1
		'Object in screen
		if objects(i, OBJECTSCREENY) = currentscreeny and objects(i, OBJECTSCREENX) = currentscreenx then
			'Is active
			if objects(i, OBJECTSTATUS) = OBJECTACTIVE then

				'Set charset to objectset
				Poke uInteger 23606, @objectset(0) - 256

				'Get the character to print
				chr1 = objectsstaticdata(objects(i, OBJECTID), OBJECTSTATICCHR)
				'Coordinates
				y = objects(i, OBJECTY)
				x = objects(i, OBJECTX)
				'Attributes
				objcolor = objects(i, OBJECTCOLOR)

				over 0

				'Depends on size x, not all objects have the same size
				l = 0
				for j = 0 to objectsstaticdata(objects(i, OBJECTID), OBJECTSTATICSIZEY) - 1
					'As many chr as size y
					for k = 0 to objectsstaticdata(objects(i, OBJECTID), OBJECTSTATICSIZEX) - 1

						'Print on screen (applying offset)
						print paper getPaper(objcolor); ink getInk(objcolor); bright getBright(objcolor); at y + j + SCREENOFFSETY, x + k + SCREENOFFSETX; chr(chr1 + l)
	
						l = l + 1
					next k
				next j

			end if
		end if
	next i

end sub

'-------------------------------------------------------------------------------------------------------------------
'Remove an object of the current screen of the map from screen when collected or used.
'-------------------------------------------------------------------------------------------------------------------
sub removeObect(objectnum as uByte)

	dim x, y, i, j as ubyte

	'Set charset to original, chr(CHRSPACE) for space is only there
	Poke uInteger 23606, 15616 - 256

	over 0

	'Coordinates
	y = objects(objectnum, OBJECTY)
	x = objects(objectnum, OBJECTX)

	'Depends on size x
	for i = 0 to objectsstaticdata(objects(objectnum, OBJECTID), OBJECTSTATICSIZEY) - 1
		'As many chr(CHRSPACE) as size y
		for j = 0 to objectsstaticdata(objects(objectnum, OBJECTID), OBJECTSTATICSIZEX) - 1

			'Remove from screen (applying offset)
			print at y + i + SCREENOFFSETY, x + j + SCREENOFFSETX; chr(CHRSPACE)
		next j
	next i

	'Make sound
    beep .001, 1

end sub

'-------------------------------------------------------------------------------------------------------------------
'Print the objects collected (keys) on scoreboard.
'In the order of collection.
'-------------------------------------------------------------------------------------------------------------------
sub showObjectsCollected()

	dim chr1, objcolor, sizex, sizey, i, j, k, l, numobject as ubyte

	numobject = 0

	'Remove all objects from objects collected scoreboard
	'In case we have used a key, so it disappears
	clearObjectsCollected()

	'And then we re-print the remaining objects collected again
	for i = 0 to NUMOBJECTS - 1

		'Is collected
		if objects(i, OBJECTSTATUS) = OBJECTCOLLECTED then

			'Set charset to objectset
			Poke uInteger 23606, @objectset(0) - 256

			'Get the character to print
			chr1 = objectsstaticdata(objects(i, OBJECTID), OBJECTSTATICCHR)
			'Attributes
			objcolor = objects(i, OBJECTCOLOR)
			'Size
			sizex = objectsstaticdata(objects(i, OBJECTID), OBJECTSTATICSIZEX)
			sizey = objectsstaticdata(objects(i, OBJECTID), OBJECTSTATICSIZEY)

			over 0

			'Depends on size x
			l = 0
			for j = 0 to sizey - 1
				'As many chr as size y
				for k = 0 to sizex - 1

					'Print on screen (applying offset)
					print paper getPaper(objcolor); ink getInk(objcolor); bright getBright(objcolor); at OBJECTSY + j + SCREENOFFSETY, OBJECTSX + (numobject*sizex) + k + SCREENOFFSETX; chr(chr1 + l)

					l = l + 1
				next k
			next j

			numobject = numobject + 1

		end if
	next i

end sub

'-------------------------------------------------------------------------------------------------------------------
'Removes the objects collected (keys) from scoreboard.
'-------------------------------------------------------------------------------------------------------------------
sub clearObjectsCollected()

	dim i, j, k, sizex, sizey as ubyte

	'Set charset to original, chr(CHRSPACE) for space is only there
	Poke uInteger 23606, 15616 - 256

	'Size of key (only object collectable)
	sizex = objectsstaticdata(OBJECTKEY, OBJECTSTATICSIZEX)
	sizey = objectsstaticdata(OBJECTKEY, OBJECTSTATICSIZEY)

	'Remove all objects
	for i = 0 to NUMOBJECTSCOLLECTABLE - 1

		'Depends on size x
		for j = 0 to sizey - 1
			'As many chr as size y
			for k = 0 to sizex - 1

				'Remove from screen (applying offset)
				print paper COLORBLACK; ink COLORBLACK; bright FALSE; at OBJECTSY + j + SCREENOFFSETY, OBJECTSX + (i*sizex) + k + SCREENOFFSETX; chr(CHRSPACE)

			next k
		next j

	next i

end sub

'-------------------------------------------------------------------------------------------------------------------
'Sets the old position of a sprite to current position so we will be able to remove it later
'-------------------------------------------------------------------------------------------------------------------
sub setOldSprite(spritenum as ubyte)

	'Old sprite new coordinates
	sprites(spritenum, SPRITEOLDY) = sprites(spritenum, SPRITEY)
	sprites(spritenum, SPRITEOLDX) = sprites(spritenum, SPRITEX)

end sub

'-------------------------------------------------------------------------------------------------------------------
'Removes old sprite from screen by printing the tile in the back
'-------------------------------------------------------------------------------------------------------------------
sub removeOldSprite(spritenum as ubyte)

	dim i, j, y, x, tile, tilechr, tilecolor as ubyte

	'If sprite has not moved, do nothing to avoid flickering
	if sprites(spritenum, SPRITEOLDX) <> sprites(spritenum, SPRITEX) or sprites(spritenum, SPRITEOLDY) <> sprites(spritenum, SPRITEY) then

		'Coordinates Y,X
		y = sprites(spritenum, SPRITEOLDY)
		x = sprites(spritenum, SPRITEOLDX)

		over 0

		'Depends on size y
		for i = 0 to spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICSIZEY) - 1
			'As many size x
			for j = 0 to spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICSIZEX) - 1

				'Get tile at position
				tile = getTileTypeAt(y + i, x + j)

				'If tile is not Front we print it, otherwise do nothing, because sprite was hiden behind
				if tile <> TILEFRONT then

					'Set charset to tileset
					Poke uInteger 23606, @tileset(0) - 256

					'Get chr and color of tile
					tilechr = getTileChrAt(y + i, x + j)
					tilecolor = getTileColorAt(y + i, x + j)

					'Print on screen (applying offset)
					print paper getPaper(tilecolor); ink getInk(tilecolor); bright getBright(tilecolor); at y + i + SCREENOFFSETY, x + j + SCREENOFFSETX; chr(tilechr)

				end if

			next j
		next i

	end if

end sub

'-------------------------------------------------------------------------------------------------------------------
'Prints all sprites on screen (Hero and enemies).
'Print is intelligent:
'If background is empty, print normally, no mask.
'If passing behind a tile, don't print, sprite is hiden.
'If passing in front of a tile, print using mask to not mix pixels, and use color of tile
'onlyHero = Means that if True, we print only the hero. Used when dying.
'-------------------------------------------------------------------------------------------------------------------
sub printSprites(onlyHero as uByte)

	dim spritenum, i, j, k, chr1, y, x, sprcolor, tile, tilecolor, totalsprites, mirror as uByte
	dim l, tilechr, bytetile, bytemask, bytesprite, byteresult as uByte
	dim chrtoprint(7) as uByte
	dim index as uInteger

	'Only hero: don't print the other sprites, it's when dying
	totalsprites = NUMSPRITES - 1
	if onlyHero = TRUE then
		totalsprites = 0
	end if

	for spritenum = 0 to totalsprites

		'Only if not Inactive
		if sprites(spritenum, SPRITESTATUS) <> SPRITEINACTIVE then

			'Get the character to print
			'It's the same for both sprite and mask
			'spriteset and maskset characters are in the same order, so we know that same chr belongs to sprite and mask
			chr1 = getChrSprite(spritenum)

			'Removes old sprite from screen
			removeOldSprite(spritenum)

			'Coordinates
			y = sprites(spritenum, SPRITEY)
			x = sprites(spritenum, SPRITEX)

			'Attributes
			sprcolor = sprites(spritenum, SPRITECOLOR)

			over 0

			k = 0
			'Depends on size y
			for i = 0 to spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICSIZEY) - 1
				'As many size x
				for j = 0 to spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICSIZEX) - 1

					'Get tile at position of chrs of the sprite, because each chr can be in a diferent tile
					tile = getTileTypeAt(y + i, x + j)

					'If tile is at back or can be trespassed we have to use mask
					if tile = TILEBACK or tile = TILESOLIDTRESPASS then

						'Get chr and color of tile
						tilechr = getTileChrAt(y + i, x + j)
						tilecolor = getTileColorAt(y + i, x + j)

						'8 rows (of 8 bits)
						for l = 0 to 7

							'Tile
							'Tilechr is known, we need just to take the byte, tileset starts at CHR 32
							'To know at what byte we are in the tileset
							'index = (8 * (tilechr - 32)) + l)
							index = tilechr - 32
							index = index * 8
							index = index + l

							bytetile = tileset(index)

							'chr used for mask and sprite
							'To know at what byte we are in the spriteset/maskset
							'index = (8 * (chr1 + k - 32)) + l
							index = chr1 + k - 32
							index = index * 8
							index = index + l

							'Spriteset/maskset for hero and enemies is in diferent set/array

							'Hero
							if spritenum = HERO then
								'Mask
								bytemask = maskheroset(index)

								'Sprite
								bytesprite = spriteheroset(index)
							'Enemies
							else
								'Mask
								bytemask = maskset(index)

								'Sprite
								bytesprite = spriteset(index)
							end if

							'Mask AND Tile
							byteresult = (bytemask bAnd bytetile)

							'Result OR Sprite
							byteresult = (byteresult bOr bytesprite)

							'Add to array row of the char of 8 x 8 bits to print
							chrtoprint(l) = byteresult

						next l

						'Set charset to chrtoprint spriteset
						Poke uInteger 23606, @chrtoprint(0) - 256

						'If tile color is black, use color of sprite, otherwise it prints black and it's invisible
						'Should not happen, just in case
						if tilecolor = COLORBLACK then
							tilecolor = sprcolor
						end if

						'Print on screen (applying offset)
						'Chr to print is always the first (32 = space) because the array has only the 8 x 8 bits we put above
						print paper getPaper(tilecolor); ink getInk(tilecolor); bright getBright(tilecolor); at y + i + SCREENOFFSETY, x + j + SCREENOFFSETX; chr(CHRSPACE)

					end if

					'If tile is empty, we print without mask, it's faster
					if tile = MAPTILEEMPTY then

						'Set charset to spriteset, different array for hero and enemies
						'Hero
						if spritenum = HERO then
							Poke uInteger 23606, @spriteheroset(0) - 256
						'Enemies
						else
							Poke uInteger 23606, @spriteset(0) - 256
						end if

						'Print on screen (applying offset)
						print paper getPaper(sprcolor); ink getInk(sprcolor); bright getBright(sprcolor); at y + i + SCREENOFFSETY, x + j + SCREENOFFSETX; chr(chr1 + k)

					end if

					k = k + 1

				next j
			next i

			'Copy old sprite vars for future removal of sprite
			setOldSprite(spritenum)
	
		end if

	next spritenum

end sub

'-------------------------------------------------------------------------------------------------------------------
'Gets the keys pressed by user and acts accordingly.
'O = Left
'P = Right
'Space = Jump
'If falling move hero down
'If jumping do the steps of the jump parabol
'-------------------------------------------------------------------------------------------------------------------
Sub moveHero ()

	dim x, y, newx, newy, sizex, sizey, status as ubyte

	'Coordinates Y,X of hero
	y = sprites(HERO, SPRITEY)
	x = sprites(HERO, SPRITEX)
	'Size of hero
	sizey = spritesstaticdata(sprites(HERO, SPRITEID), SPRITESTATICSIZEY)
	sizex = spritesstaticdata(sprites(HERO, SPRITEID), SPRITESTATICSIZEX)

	'---------------------------------------------------------------------------------------------------------------------
	'Sprite is Active
	'---------------------------------------------------------------------------------------------------------------------
    if sprites(HERO, SPRITESTATUS) = SPRITEACTIVE then

		'If no floor then fall
	    if ((isSolidFloor(y + sizey, x) = FALSE and isSolidFloor(y + sizey, x + sizex - 1) = FALSE) _
	    	or y = ((MAPSCREENHEIGHT * TILESIZE) - sizey)) then
	    	
	    	sprites(HERO, SPRITESTATUS) = SPRITEFALLING
	    	sprites(HERO, SPRITEDIRY) = DIRDOWN
	    
	    'If there is floor under hero
	    else

	    	'---------------------------------------------------------------------------------------------------------------------
		    'JUMP: Detectar "SP" (Space): Octava semifila, bit 0
	    	'---------------------------------------------------------------------------------------------------------------------
		    If (In (32766) bAnd 1) = 0 Then 

		    	'Reset jump and change status
		    	resetJump()
		    	sprites(HERO, SPRITESTATUS) = SPRITEJUMPING

		    	'Sound of jump
		    	beep .02, 3
		    
		    End If

	    	'---------------------------------------------------------------------------------------------------------------------
		    'RIGHT: Detectar "P": Sexta semifila, bit 0
	    	'---------------------------------------------------------------------------------------------------------------------
		    If (In (57342) bAnd 1) = 0 Then

		        'Even if not moving if user clicked right while jumping change direction.
		    	if sprites(HERO, SPRITESTATUS) = SPRITEJUMPING then
			        jumpDirection = DIRRIGHT

			    'If not jumping, then walk
		        else

		        	'Change status to active, in case it was falling or in other status
			        sprites(HERO, SPRITEDIRY) = DIRNONE
			    	sprites(HERO, SPRITESTATUS) = SPRITEACTIVE

			    	'Detect border screen
			    	'If still inside screen
				    if x < ((MAPSCREENWIDTH * TILESIZE) - sizex) then
				        
				    	'Increment X
				        newx = x + MOVECHR
				        newy = y
				        
				    	'Detect collision with tiles right
				    	'If no collision
				        if getTileTypeAt(newy, newx + sizex - 1) <> TILESOLID and getTileTypeAt(newy + sizey - 1, newx + sizex - 1) <> TILESOLID then

				        	'Apply new position
					        sprites(HERO, SPRITEX) = newx
					        'If was looking left, change to right
					        changeDirectionX(HERO,DIRRIGHT)

					        'Sound of step
					        beep .001, 1
				        
				        end if
					        
			        'If we were in border of screen
			        else

				        'If was looking left, change to right
				        changeDirectionX(HERO,DIRRIGHT)
			        	'Move to next screen right
			        	changeScreen(DIRRIGHT)

			        end if
		        end if

	        end if

	    	'---------------------------------------------------------------------------------------------------------------------
		    'LEFT: Detectar "O": Sexta semifila, bit 1
	    	'---------------------------------------------------------------------------------------------------------------------
			If (In (57342) bAnd 2) = 0 Then 

		        'Even if not moving if user clicked left while jumping change direction.
		    	if sprites(HERO, SPRITESTATUS) = SPRITEJUMPING then
			        jumpDirection = DIRLEFT
		        else

		        	'Change status to active, in case it was falling or in other status
			        sprites(HERO, SPRITEDIRY) = DIRNONE
			    	sprites(HERO, SPRITESTATUS) = SPRITEACTIVE

			    	'Detect border screen
			    	'If still inside screen
				    if x > 0 then
				        
				    	'Decrement X
				        newx = x - MOVECHR
				        newy = y

				    	'Detect collision with tiles left
				    	'If no collision
				        if getTileTypeAt(newy, newx) <> TILESOLID and getTileTypeAt(newy + sizey - 1, newx) <> TILESOLID then

				        	'Apply new position
					        sprites(HERO, SPRITEX) = newx
					        'If was looking right, change to left
					        changeDirectionX(HERO,DIRLEFT)

					        'Sound of step
					        beep .001, 1
				        
				        end if
					        
			        'If we were in border of screen
			        else
				        'If was looking right, change to left
				        changeDirectionX(HERO,DIRLEFT)
			        	'Move to next screen left
			        	changeScreen(DIRLEFT)        
			        end if
		        end if

	        end if

       	end if

	'---------------------------------------------------------------------------------------------------------------------
	'Sprite is Jumping
	'---------------------------------------------------------------------------------------------------------------------
    elseif sprites(HERO, SPRITESTATUS) = SPRITEJUMPING then

    	'Y direction - Jump:
    	'-1: up
    	if jumpStepsY(jumpStep) < 0 then
    		'Direction UP
    		sprites(HERO, SPRITEDIRY) = DIRUP
    		'First frame jump
			sprites(HERO, SPRITEFRAMEJUMP) = SPRITEFIRSTFRAME
    	'1: down
    	else if jumpStepsY(jumpStep) > 0 then
    		'Direction DOWN
    		sprites(HERO, SPRITEDIRY) = DIRDOWN
    		'Second frame jump
			sprites(HERO, SPRITEFRAMEJUMP) = SPRITEFIRSTFRAME + 1
    	'0: None
	    else
    		'Direction NONE
    		sprites(HERO, SPRITEDIRY) = DIRNONE
    		'First frame jump
			sprites(HERO, SPRITEFRAMEJUMP) = SPRITEFIRSTFRAME
    	end if

    	'Detect top of the screen or top of the map
    	'If still inside screen or we are in top border of top screen of map or going down
	    if y > 0 or (y = 0 and (currentscreeny = 0 or sprites(HERO, SPRITEDIRY) = DIRDOWN)) then

	    	'By default don't move
	    	newy = y
	    	'If it can move then do
	        if (y > 0 AND sprites(HERO, SPRITEDIRY) = DIRUP) or _
	        	sprites(HERO, SPRITEDIRY) = DIRDOWN or _
	        	sprites(HERO, SPRITEDIRY) = DIRNONE then
		        
	        	'Increment/Decrement Y position (depending on jump step)
		        newy = y + jumpStepsY(jumpStep)
		    end if

	    	'If moving right
	        if jumpDirection = DIRRIGHT then

		    	'Detect border screen
		    	'If still inside screen
			    if x < ((MAPSCREENWIDTH * TILESIZE) - sizex) then

		        	'Increment or not X position (depending on jump step)
		        	newx = x + jumpStepsX(jumpStep)
			        
		        	'If moving Up
			        if sprites(HERO, SPRITEDIRY) = DIRUP then

				    	'Detect collision with tiles right+up
				    	'If no collision
				        if getTileTypeAt(newy, newx + sizex - 1) <> TILESOLID and getTileTypeAt(newy + sizey - 1, newx + sizex - 1) <> TILESOLID _
					        and getTileTypeAt(newy, newx) <> TILESOLID then

				        	'Apply new position
					        sprites(HERO, SPRITEX) = newx
					        sprites(HERO, SPRITEY) = newy

					    end if

					'If moving Down
			        elseif sprites(HERO, SPRITEDIRY) = DIRDOWN then

				    	'Detect bottom screen
				    	'If getting out by bottom
					    if newy > ((MAPSCREENHEIGHT * TILESIZE) - sizey) then

				        	'Apply new position X
					        sprites(HERO, SPRITEX) = newx
				        	'Change to next screen down
				        	changeScreen(DIRDOWN)

				    	'Detect collision with tiles right
				    	'If no collision
				        else if getTileTypeAt(newy, newx + sizex) <> TILESOLID then

					    	'Detect collision with tiles down
					    	'If no collision
				        	if isSolidFloor(newy + sizey - 1, newx + sizex - 1) = FALSE and isSolidFloor(newy + sizey - 1, newx) = FALSE then

					        	'Apply new position
						        sprites(HERO, SPRITEX) = newx
						        sprites(HERO, SPRITEY) = newy

								'If in deep water (Tile Die exist only in water)
							    if getTileTypeAt(newy + sizey - 1, newx) = TILEDIE and getTileTypeAt(newy + sizey - 1, newx + sizex - 1) = TILEDIE then
							    	'Collided
							    	collided()
							    end if

						    'Touched ground
						    else
						    	'Stop jumping
						    	resetJump()
						    end if

					    'Touched ground
					    else
					    	'Stop jumping
					    	resetJump()
					    end if

					'If not moving
					else

				    	'Detect collision with tiles right
				    	'If no collision
				        if getTileTypeAt(newy, newx + sizex - 1) <> TILESOLID and getTileTypeAt(newy + sizey - 1, newx + sizex - 1) <> TILESOLID _
					        and getTileTypeAt(newy, newx) <> TILESOLID then

				        	'Apply new position
					        sprites(HERO, SPRITEX) = newx
					        sprites(HERO, SPRITEY) = newy

					    'Touched ground
					    else
					    	'Stop jumping
					    	resetJump()
					    end if
				    end if

				'In right border of screen
				else
		        	'Change to next screen right
		        	changeScreen(DIRRIGHT)
		    	end if

	    	'If moving left 
	        elseif jumpDirection = DIRLEFT then

		    	'Detect border screen
		    	'If still inside screen
			    if x > 0 then

		        	'Decrement or not X position (depending on jump step)
		        	newx = x - jumpStepsX(jumpStep)

		        	'If moving Up
			        if sprites(HERO, SPRITEDIRY) = DIRUP then

				    	'Detect collision with tiles left+up
				    	'If no collision
				        if getTileTypeAt(newy, newx) <> TILESOLID and getTileTypeAt(newy + sizey - 1, newx) <> TILESOLID _
					        and getTileTypeAt(newy, newx + sizex - 1) <> TILESOLID then

				        	'Apply new position
					        sprites(HERO, SPRITEX) = newx
					        sprites(HERO, SPRITEY) = newy

					    end if

					'If moving Down
			        elseif sprites(HERO, SPRITEDIRY) = DIRDOWN then

				    	'Detect bottom screen
				    	'If getting out by bottom
					    if newy > ((MAPSCREENHEIGHT * TILESIZE) - sizey) then

				        	'Apply new position X
					        sprites(HERO, SPRITEX) = newx
				        	'Change to next screen down
				        	changeScreen(DIRDOWN)

				    	'Detect collision with tiles left
				    	'If no collision
				        else if getTileTypeAt(newy, newx) <> TILESOLID then

					    	'Detect collision with tiles down
					    	'If no collision
				        	if isSolidFloor(newy + sizey - 1, newx + sizex - 1) = FALSE and isSolidFloor(newy + sizey - 1, newx) = FALSE then

					        	'Apply new position
						        sprites(HERO, SPRITEX) = newx
						        sprites(HERO, SPRITEY) = newy
								
								'If in deep water (Tile Die exist only in water)
							    if getTileTypeAt(newy + sizey - 1, newx) = TILEDIE and getTileTypeAt(newy + sizey - 1, newx + sizex - 1) = TILEDIE then
							    	'Collided
							    	collided()
							    end if

						    'Touched ground
						    else
						    	'Stop jumping
						    	resetJump()
						    end if

					    'Touched ground
					    else
					    	'Stop jumping
					    	resetJump()
					    end if

					'If not moving
					else

				    	'Detect collision with tiles left+up
				    	'If no collision
				        if getTileTypeAt(newy, newx) <> TILESOLID and getTileTypeAt(newy + sizey - 1, newx) <> TILESOLID _
					        and getTileTypeAt(newy, newx + sizex - 1) <> TILESOLID then

				        	'Apply new position
					        sprites(HERO, SPRITEX) = newx
					        sprites(HERO, SPRITEY) = newy

					    'Touched ground
					    else
					    	'Stop jumping
					    	resetJump()
					    end if
				    end if

				'In left border of screen
				else
		        	'Change to next screen left
		        	changeScreen(DIRLEFT)
		    	end if

	    	'If not moving (vertical jump)
	        elseif jumpDirection = DIRNONE then

	        	'If moving Up
		        if sprites(HERO, SPRITEDIRY) = DIRUP then

			    	'Detect collision with tiles up
			    	'If no collision
			        if getTileTypeAt(newy, x) <> TILESOLID and getTileTypeAt(newy + sizey - 1, x) <> TILESOLID _
				        and getTileTypeAt(newy, x + sizex - 1) <> TILESOLID then
	
			        	'Apply new position Y
				        sprites(HERO, SPRITEY) = newy
				    end if

				'If moving Down
		        else if sprites(HERO, SPRITEDIRY) = DIRDOWN then

			    	'Detect bottom screen
			    	'If getting out by bottom
				    if newy > ((MAPSCREENHEIGHT * TILESIZE) - sizey) then

			        	'Change to next screen down
			        	changeScreen(DIRDOWN)
			    	
			    	'Detect collision with tiles down
			    	'If no collision
				    else if getTileTypeAt(newy, x) <> TILESOLID and isSolidFloor(newy + sizey - 1, x) = FALSE _
				    	and isSolidFloor(newy + sizey - 1, x + sizex - 1) = FALSE then

			        	'Apply new position Y
				        sprites(HERO, SPRITEY) = newy

				    'Touched ground
				    else
				    	'Stop jumping
				    	resetJump()
				    end if

				'If not moving
				else
			    	'No need to verify, hero does not move
			    end if

	        end if

	    'In top border of screen
        else

        	'If moving Up
        	if sprites(HERO, SPRITEDIRY) = DIRUP then

	        	'Change to next screen up
	        	changeScreen(DIRUP)

        	'If moving Down
	        else

		    	'Detect bottom screen
		    	'If getting out by bottom
			    if y >= ((MAPSCREENHEIGHT * TILESIZE) - sizey) then

		        	'Change to next screen down
		        	changeScreen(DIRDOWN)

			    end if
		    end if
	    end if

	    'If last jump step
    	if jumpStep = JUMPSTEPS -1 then
    		'Stop jumping
    		resetJump()
    	'If not
		else
	    	'Increase jump step
	    	jumpStep = jumpStep + 1
	    end if

	'---------------------------------------------------------------------------------------------------------------------
	'Sprite is Falling
	'---------------------------------------------------------------------------------------------------------------------
    elseif sprites(HERO, SPRITESTATUS) = SPRITEFALLING then

    	'Detect bottom of screen
    	'If still inside screen
	    if y < (MAPSCREENHEIGHT * TILESIZE - sizey) then

        	'Increment Y position
	    	newx = x
	    	newy = sprites(HERO, SPRITEY) + 1

			'If no floor then continue falling
		    if isSolidFloor(newy + sizey - 1, newx) = FALSE and isSolidFloor(newy + sizey - 1, newx + sizex - 1) = FALSE then

	        	'Apply new position Y
		    	sprites(HERO, SPRITEY) = newy
		    	'Increase fall counter
			    fallCount = fallCount + 1

				'If in deep water (Tile Die exist only in water)
			    if getTileTypeAt(newy + sizey - 1, newx) = TILEDIE and getTileTypeAt(newy + sizey - 1, newx + sizex - 1) = TILEDIE then
			    	'Collided
			    	collided()
			    end if

			'If touched floor
		    else

		    	'If max height fall then die
		    	if fallCount >= FALLHIGHMAX then
			    	'Collided
			    	collided()
		    	end if

		    	'Otherwise stop jumping safely
		    	resetJump()

		    end if

    	'If getting out by bottom
        else

	    	'Increase fall counter
		    fallCount = fallCount + 1
        	'Change to next screen down
        	changeScreen(DIRDOWN)

	    end if

    End If

End Sub

'-------------------------------------------------------------------------------------------------------------------
'Change horizontal direction of enemy and increments the frame if necessary.
'If it was looking left and now moves right change direction to right and vice-versa
'-------------------------------------------------------------------------------------------------------------------
sub changeDirectionX(spritenum as ubyte, newDirection as ubyte)

	'Horizontal
	'If not moving
    if sprites(spritenum, SPRITEDIRX) = DIRNONE then

    	'Increments frame
        nextFrame(spritenum)
        'Change direction
        sprites(spritenum, SPRITEDIRX) = newDirection

	'If moving in the good direction
   elseif sprites(spritenum, SPRITEDIRX) = newDirection then

    	'Increments frame
        nextFrame(spritenum)

	'If moving in the opposite direction 
    else

    	'Resets frame
    	sprites(spritenum, SPRITEFRAME) = SPRITEFIRSTFRAME
        'Change direction
        sprites(spritenum, SPRITEDIRX) = newDirection

    end if

end sub

'-------------------------------------------------------------------------------------------------------------------
'Change vertical direction of enemy and increments the frame if necessary.
'If it was looking up and now moves down change direction to down and vice-versa
'-------------------------------------------------------------------------------------------------------------------
sub changeDirectionY(spritenum as ubyte, newDirection as ubyte)

	'If not moving
    if sprites(spritenum, SPRITEDIRY) = DIRNONE then

    	'Increments frame
        nextFrame(spritenum)
        'Change direction
        sprites(spritenum, SPRITEDIRY) = newDirection

	'If moving in the good direction
    elseif sprites(spritenum, SPRITEDIRY) = newDirection then

    	'Increments frame
        nextFrame(spritenum)

	'If moving in the opposite direction
    else

    	'Resets frame
    	sprites(spritenum, SPRITEFRAME) = SPRITEFIRSTFRAME
        'Change direction
        sprites(spritenum, SPRITEDIRY) = newDirection

    end if

end sub

'-------------------------------------------------------------------------------------------------------------------
'Change frame and loops.
'-------------------------------------------------------------------------------------------------------------------
sub nextFrame(spritenum as ubyte)
	
	'If last frame
    if sprites(spritenum, SPRITEFRAME) = spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICFRAMES) - 1 then
    	'Reset frame
    	sprites(spritenum, SPRITEFRAME) = SPRITEFIRSTFRAME
    'If not
	else
    	'Increments frame
        sprites(spritenum, SPRITEFRAME) = sprites(spritenum, SPRITEFRAME) + 1
    end if	
End Sub

'-------------------------------------------------------------------------------------------------------------------
'Moves the enemies depending on their type.
'-------------------------------------------------------------------------------------------------------------------
sub moveEnemies()
	
	dim i as ubyte

	'Loops all enemies on screen (only 1 in this game)
	for i = 1 to NUMSPRITES - 1

		'--------------------------
		'ENEMYHORIZONTALFLOOR
		'--------------------------
		if spritesstaticdata(sprites(i, SPRITEID), SPRITESTATICTYPE) = ENEMYHORIZONTALFLOOR then
			moveEnemyHorizontalFloor(i)

		'--------------------------
		'ENEMYFLY
		'--------------------------
		elseif spritesstaticdata(sprites(i, SPRITEID), SPRITESTATICTYPE) = ENEMYFLY then
			moveEnemyFly(i)

		'--------------------------
		'ENEMYSTALACTITE
		'--------------------------
		elseif spritesstaticdata(sprites(i, SPRITEID), SPRITESTATICTYPE) = ENEMYSTALACTITE then
			moveEnemyStalactite(i)

		'--------------------------
		'ENEMYFISH
		'--------------------------
		elseif spritesstaticdata(sprites(i, SPRITEID), SPRITESTATICTYPE) = ENEMYFISH then
			moveEnemyFish(i)
		end if

	next i

end sub

'-------------------------------------------------------------------------------------------------------------------
'ENEMYHORIZONTALFLOOR:
'Moves horizontally on the floor. Turns when reaching its min or max limits of movement or when colliding with borders, walls and holes.
'-------------------------------------------------------------------------------------------------------------------
sub moveEnemyHorizontalFloor(spritenum as ubyte)

	dim x, y, newx, newy, sizex, sizey as ubyte

	'Coordinates
	y = sprites(spritenum, SPRITEY)
	x = sprites(spritenum, SPRITEX)
	'Size
	sizex = spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICSIZEX)
	sizey = spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICSIZEY)

	'If sprite is Active
    if sprites(spritenum, SPRITESTATUS) = SPRITEACTIVE then

		'If moving right
	    if sprites(spritenum, SPRITEDIRX) = DIRRIGHT then

	    	'Detect border screen
	    	'If still inside screen
		    if x < ((MAPSCREENWIDTH * TILESIZE) - sizex) then

		    	'Detect max x
		    	'If limit not reached
			    if x < sprites(spritenum, SPRITEMAXX) then

			    	'Increment X position
			        newx = x + MOVECHR
			        newy = y
			        
			    	'Detect collision with tiles right and floor
			    	'If no collision
			        if getTileTypeAt(newy, newx + sizex - 1) <> TILESOLID and getTileTypeAt(newy + sizey - 1, newx + sizex - 1) <> TILESOLID then

			        	'Apply new position X
				        sprites(spritenum, SPRITEX) = newx
				        'If it was looking right, stay right
				        changeDirectionX(spritenum, DIRRIGHT)

			        'If collision
			        else
				        'If it was looking right, change to left
				        changeDirectionX(spritenum, DIRLEFT)
			        end if
			        
			    'If limit reached
		        else
			        'If it was looking right, change to left
			        changeDirectionX(spritenum, DIRLEFT)
			    end if
	        
	        'Border of screen
	        else
		        'If it was looking right, change to left
		        changeDirectionX(spritenum, DIRLEFT)
	        end if

		'If moving left
	    elseif sprites(spritenum, SPRITEDIRX) = DIRLEFT then

	    	'Detect border screen
	    	'If still inside screen
		    if x > 0 then

		    	'Detect min x
		    	'If limit not reached
			    if x > sprites(spritenum, SPRITEMINX) then

			    	'Decrement X position
			        newx = x - MOVECHR
			        newy = y

			    	'Detect collision with tiles left and floor
			    	'If no collision
			        if getTileTypeAt(newy, newx) <> TILESOLID and getTileTypeAt(newy + sizey - 1, newx) <> TILESOLID then

			        	'Apply new position X
				        sprites(spritenum, SPRITEX) = newx
				        'If it was looking left, stay left
				        changeDirectionX(spritenum, DIRLEFT)

			        'If collision
			        else
				        'If it was looking left, change to right
				        changeDirectionX(spritenum, DIRRIGHT)
			        end if
			        
			    'If limit reached
		        else
			        'If it was looking left, change to right
			        changeDirectionX(spritenum, DIRRIGHT)
		        end if

	        'Border of screen
	        else
		        'If it was looking left, change to right
		        changeDirectionX(spritenum, DIRRIGHT)
	        end if

	    end if

    end if

end sub

'-------------------------------------------------------------------------------------------------------------------
'ENEMYFLY:
'Moves horizontally/vertically on the air. Turns when reaching its min or max limits of movement or when colliding with borders and walls.
'-------------------------------------------------------------------------------------------------------------------
sub moveEnemyFly(spritenum as ubyte)

	dim x, y, newx, newy, sizex, sizey, addedFrame as ubyte

	addedFrame = FALSE

	'Coordinates
	y = sprites(spritenum, SPRITEY)
	x = sprites(spritenum, SPRITEX)
	'Size
	sizey = spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICSIZEY)
	sizex = spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICSIZEX)
	newy = y
	newx = x

	'If sprite is Active
    if sprites(spritenum, SPRITESTATUS) = SPRITEACTIVE then

    	'-------------------------------------
    	'Horizontal
    	'-------------------------------------
		'If moving right
	    if sprites(spritenum, SPRITEDIRX) = DIRRIGHT then

	    	'Detect border screen
	    	'If still inside screen
		    if x < ((MAPSCREENWIDTH * TILESIZE) - sizex) then

		    	'Detect max x
		    	'If limit not reached
			    if x < sprites(spritenum, SPRITEMAXX) then

			    	'Increment X position
			        newx = x + MOVECHR

			    	'Detect collision with tiles right
			    	'If no collision
			        if getTileTypeAt(newy, newx + sizex - 1) <> TILESOLID and getTileTypeAt(newy + sizey - 1, newx + sizex - 1) <> TILESOLID then

			        	'Apply new position X
				        sprites(spritenum, SPRITEX) = newx
				        'If it was looking right, stay right
				        changeDirectionX(spritenum, DIRRIGHT)
				        addedFrame = TRUE

			        'If collision
			        else
				        'If it was looking right, change to left
				        changeDirectionX(spritenum, DIRLEFT)
			        end if
			        
			    'If limit reached
		        else
			        'If it was looking right, change to left
			        changeDirectionX(spritenum, DIRLEFT)
		        end if

	        'Border of screen
	        else
		        'If it was looking right, change to left
		        changeDirectionX(spritenum, DIRLEFT)
	        end if

		'If moving left
	    elseif sprites(spritenum, SPRITEDIRX) = DIRLEFT then

	    	'Detect border screen
	    	'If still inside screen
		    if x > 0 then

		    	'Detect min x
		    	'If limit not reached
			    if x > sprites(spritenum, SPRITEMINX) then

			    	'Decrement X position
			        newx = x - MOVECHR

			    	'Detect collision with tiles left and floor
			    	'If no collision
			        if getTileTypeAt(newy, newx) <> TILESOLID and getTileTypeAt(newy + sizey - 1, newx) <> TILESOLID then

			        	'Apply new position X
				        sprites(spritenum, SPRITEX) = newx
				        'If it was looking left, stay left
				        changeDirectionX(spritenum, DIRLEFT)
				        addedFrame = TRUE

			        'If collision
			        else
				        'If it was looking left, change to right
				        changeDirectionX(spritenum, DIRRIGHT)
			        end if
			        
			    'If limit reached
		        else
			        'If it was looking left, change to right
			        changeDirectionX(spritenum, DIRRIGHT)
		        end if

	        'Border of screen
	        else
		        'If it was looking left, change to right
		        changeDirectionX(spritenum, DIRRIGHT)
	        end if

	    end if

    	'-------------------------------------
    	'Vertical
    	'-------------------------------------
		'If moving up
	    if sprites(spritenum, SPRITEDIRY) = DIRUP then

	    	'Detect border screen
	    	'If still inside screen
		    if y > 0 then

		    	'Detect min y
		    	'If limit not reached
			    if y > sprites(spritenum, SPRITEMINY) then

			    	'Decrement X position
			        newy = y - MOVECHR

			    	'Detect collision with tiles ceiling
			    	'If no collision
			        if getTileTypeAt(newy, newx) <> TILESOLID and getTileTypeAt(newy, newx + sizex - 1) <> TILESOLID then

			        	'Apply new position Y
				        sprites(spritenum, SPRITEY) = newy
				        'If it was looking up, stay up
				        if (addedFrame = FALSE) then
					        changeDirectionY(spritenum, DIRUP)
				        end if

			        'If collision
			        else
				        'If it was looking up, change to down
				        changeDirectionY(spritenum, DIRDOWN)
			        end if
			        
			    'If limit reached
		        else
			        'If it was looking up, change to down
			        changeDirectionY(spritenum, DIRDOWN)
		        end if

	        'Border of screen
	        else
		        'If it was looking up, change to down
		        changeDirectionY(spritenum, DIRDOWN)
	        end if

		'If moving down
	    elseif sprites(spritenum, SPRITEDIRY) = DIRDOWN then

	    	'Detect border screen
	    	'If still inside screen
		    if y < ((MAPSCREENHEIGHT * TILESIZE) - sizey) then

		    	'Detect max y
		    	'If limit not reached
			    if y < sprites(spritenum, SPRITEMAXY) then

			    	'Increment X position
			        newy = y + MOVECHR

			    	'Detect collision with floor
			    	'If no collision
			        if getTileTypeAt(newy + sizey - 1, newx) <> TILESOLID and getTileTypeAt(newy + sizey - 1, newx + sizex - 1) <> TILESOLID then

			        	'Apply new position Y
				        sprites(spritenum, SPRITEY) = newy
				        'If it was looking down, stay down
				        if (addedFrame = FALSE) then
					        changeDirectionY(spritenum, DIRDOWN)
				        end if

			        'If collision
			        else
				        'If it was looking down, change to up
				        changeDirectionY(spritenum, DIRUP)
			        end if
			        
			    'If limit reached
		        else
			        'If it was looking down, change to up
			        changeDirectionY(spritenum, DIRUP)
		        end if

	        'Border of screen
	        else
		        'If it was looking down, change to up
		        changeDirectionY(spritenum, DIRUP)
	        end if

	    end if

    end if

end sub

'-------------------------------------------------------------------------------------------------------------------
'ENEMYSTALACTITE:
'Falls from ceiling. Crashes when colliding with floor.
'2 frames when forming, 1 frame when falling and 1 frame when crashing.
'-------------------------------------------------------------------------------------------------------------------
sub moveEnemyStalactite(spritenum as ubyte)

	dim x, y, newy, sizey, sizex, frame as ubyte

	'Coordinates
	y = sprites(spritenum, SPRITEY)
	x = sprites(spritenum, SPRITEX)
	'Size
	sizey = spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICSIZEY)
	sizex = spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICSIZEX)
	'Frame
	frame = sprites(spritenum, SPRITEFRAME)

	'If sprite is Active
    if sprites(spritenum, SPRITESTATUS) = SPRITEACTIVE then

    	'If appearing (frames 0 and 1 are static on ceiling, when forming, not move)
	    if frame <= 1 then

	    	'Increment frame
	    	nextFrame(spritenum)
    	
    	'If falling (frame 2 is always the same when falling till it touches floor)
	    elseif frame = 2 then

	    	'Detect bottom screen
	    	'If border of screen
		    if y = ((MAPSCREENHEIGHT * TILESIZE) - spritesstaticdata(sprites(spritenum, SPRITEID), SPRITESTATICSIZEY)) then

		    	'Reinitialise loop
		    	resetEnemyStalactite(spritenum)

	    	'If still inside screen
		    else

		    	'Increment position Y
		        newy = y + MOVECHR
			        
		    	'Detect collision with tiles down
		    	'If no collision
		        if getTileTypeAt(newy + sizey - 1, x) <> TILESOLID and getTileTypeAt(newy + sizey - 1, x + sizex - 1) <> TILESOLID then

		        	'Apply new position Y
			        sprites(spritenum, SPRITEY) = newy

			    'If collision
		        else

			    	'Increment frame
			    	nextFrame(spritenum)
		
		        end if
		        
	        end if

    	'If crashing with floor (frame 3 it touched floor and explodes)
	    elseif frame = 3 then

	    	'Reinitialise loop
	    	resetEnemyStalactite(spritenum)

	    end if

    end if

end sub

'-------------------------------------------------------------------------------------------------------------------
'Reinitialises stalactite.
'Resets frame and sets Y position to initial.
'-------------------------------------------------------------------------------------------------------------------
sub resetEnemyStalactite (spritenum as uByte)

	'Resets frame
	sprites(spritenum, SPRITEFRAME) = SPRITEFIRSTFRAME
	'Resets Y position
	sprites(spritenum, SPRITEY) = sprites(spritenum, SPRITEINITY)

end sub

'-------------------------------------------------------------------------------------------------------------------
'ENEMYFISH:
'Moves vertically up/down. Changes direction when reaching Min or Max limits.
'Same sprites are used to go up and down, no turning.
'There is no verification of borders of screen or collisions with floor/ceiling, 
'when defining the limits we take care to put them in correct places.
'-------------------------------------------------------------------------------------------------------------------
sub moveEnemyFish(spritenum as ubyte)

	dim y as ubyte

	'Coordinates
	y = sprites(spritenum, SPRITEY)

	'If sprite is Active
    if sprites(spritenum, SPRITESTATUS) = SPRITEACTIVE then

		'If moving up
	    if sprites(spritenum, SPRITEDIRY) = DIRUP then

	    	'Detect min Y
	    	'Limit not reached
		    if y > sprites(spritenum, SPRITEMINY) then

		    	'Decrement position Y and apply
		        sprites(spritenum, SPRITEY) = y - MOVECHR
		        
		        'If it was looking up, stay up
		        changeDirectionY(spritenum, DIRUP)
	        
		    'Limit reached
	        else
		        'If it was looking up, change to down
		        changeDirectionY(spritenum, DIRDOWN)
	        end if


		'If moving down
	    elseif sprites(spritenum, SPRITEDIRY) = DIRDOWN then

	    	'Detect max Y
	    	'Limit not reached
		    if y < sprites(spritenum, SPRITEMAXY) then

		    	'Increment position Y and apply
		        sprites(spritenum, SPRITEY) = y + MOVECHR

		        'If it was looking down, stay down
		        changeDirectionY(spritenum, DIRDOWN)
	        
		    'Limit reached
	        else
		        'If it was looking down, change to up
		        changeDirectionY(spritenum, DIRUP)
	        end if

	    end if

    end if

end sub

'-------------------------------------------------------------------------------------------------------------------
'Detectcs collision of hero with enemies
'-------------------------------------------------------------------------------------------------------------------
sub detectCollisionSprites()

	dim i, sizeherox, sizeheroy, sizeenemyx, sizeenemyy as ubyte
	dim x, y as ubyte

	'Coordinates
	x = sprites(HERO, SPRITEX)
	y = sprites(HERO, SPRITEY)
	'Size of hero
	sizeherox = spritesstaticdata(sprites(HERO, SPRITEID), SPRITESTATICSIZEX)
	sizeheroy = spritesstaticdata(sprites(HERO, SPRITEID), SPRITESTATICSIZEY)

	'Loops all enemies
	for i = 1 to NUMSPRITES - 1

		'If sprite is Active
	    if sprites(i, SPRITESTATUS) = SPRITEACTIVE then

			'Size of enemy
			sizeenemyx = spritesstaticdata(sprites(i, SPRITEID), SPRITESTATICSIZEX)
			sizeenemyy = spritesstaticdata(sprites(i, SPRITEID), SPRITESTATICSIZEY)

			'If collided
			if sprites(i, SPRITEX) + sizeenemyx - 1 >= x and sprites(i, SPRITEX) <= x + sizeherox - 1 _
				and sprites(i, SPRITEY) + sizeenemyy - 1 >= y and sprites(i, SPRITEY) <= y + sizeheroy - 1 then

				'Collided
				collided()

			end if

		end if

	next i

end sub

'-------------------------------------------------------------------------------------------------------------------
'Hero has collided with an enemy or deep water or fallen from too high
'Die and update lives. If no more lives then Game Over.
'-------------------------------------------------------------------------------------------------------------------
sub collided()
	
	'Dying
	die()

	'If still has lives
	if lives > 0 then
		'Decrease lives counter
		lives = lives - 1
	end if

	'Show lives in scoreboard
	showLives()

	'If no more lives
	if lives = 0 then
		'Game over
		gameover = TRUE
	end if

	'If not game over
	if gameover = FALSE then
	
		'Pause does not work if you clicked a key during the game, they are still in memory
		'This is dirty, but it works to empty the keystrokes
		dim i as uInteger
		for i = 0 to 30000
		next i

		'Show get ready message
		getReady()
		'Reset fall and jump
		fallCount = 0
    	resetJump()
    	'Init the screen and start again
    	'TRUE - to signal that we have died
		initScreen(currentscreeny, currentscreenx, TRUE)

	end if

end sub

'-------------------------------------------------------------------------------------------------------------------
'Hero dies.
'Print the 2 frames of dying.
'-------------------------------------------------------------------------------------------------------------------
sub die()
	
	'Change status to Dying
	sprites(HERO, SPRITESTATUS) = SPRITEDYING
	'Print first frame of dying
	sprites(HERO, SPRITEFRAMEDIE) = SPRITEFIRSTFRAME
	'TRUE - Print only the Hero, not the enemies, they are static
	printSprites(TRUE)

	'Pause does not work if you clicked a key during the game, they are still in memory
	'This is dirty, but it works to empty the keystrokes
	dim i as uInteger
	for i = 0 to 10000
	next i

	'Print second frame of dying
	sprites(HERO, SPRITEFRAMEDIE) = sprites(HERO, SPRITEFRAMEDIE) + 1
	printSprites(TRUE)

end sub

'-------------------------------------------------------------------------------------------------------------------
'Detectcs collision of hero with objects.
'Objects are:
'Live: Increment lives counter.
'Key: Adds it to objects collected.
'Door: If we have the correct key, opens the door.
'Final (Princess): If we have the correct key, opens the cage and frees the princess. End game!
'-------------------------------------------------------------------------------------------------------------------
sub detectCollisionObjects()

	dim x, y, i, sizeherox, sizeheroy, sizeobjectx, sizeobjecty as ubyte

	'Coordinates
	x = sprites(HERO, SPRITEX)
	y = sprites(HERO, SPRITEY)
	'Size of hero
	sizeherox = spritesstaticdata(sprites(HERO, SPRITEID), SPRITESTATICSIZEX)
	sizeheroy = spritesstaticdata(sprites(HERO, SPRITEID), SPRITESTATICSIZEY)

	'Loops all objects in map
	for i = 0 to NUMOBJECTS - 1

		'If Object is in screen
		if objects(i, OBJECTSCREENY) = currentscreeny and objects(i, OBJECTSCREENX) = currentscreenx then

			'If object is Active
		    if objects(i, OBJECTSTATUS) = OBJECTACTIVE then

		    	'Size of object
				sizeobjectx = objectsstaticdata(objects(i, OBJECTID), OBJECTSTATICSIZEX)
				sizeobjecty = objectsstaticdata(objects(i, OBJECTID), OBJECTSTATICSIZEY)

		    	'If collided
				if objects(i, OBJECTX) + sizeobjectx - 1 >= x and objects(i, OBJECTX) <= x + sizeherox - 1 _
					and objects(i, OBJECTY) + sizeobjecty - 1 >= y and objects(i, OBJECTY) <= y + sizeheroy - 1 then

					'-------------------------------------------------------------------------------------------------------------------
					'Life
					'-------------------------------------------------------------------------------------------------------------------
					if objectsstaticdata(objects(i, OBJECTID), OBJECTSTATICTYPE) = OBJECTLIFE then
					
						'Removes object from screen
						removeObect(i)
						'Add new live
						lives = lives + 1
						'Refresh lives counter in scoreboard
						showLives()
						'Inactivate live object
						objects(i, OBJECTSTATUS) = OBJECTINACTIVE

					'-------------------------------------------------------------------------------------------------------------------
					'Key
					'-------------------------------------------------------------------------------------------------------------------
					elseif objectsstaticdata(objects(i, OBJECTID), OBJECTSTATICTYPE) = OBJECTKEY then
					
						'Removes object from screen
						removeObect(i)
						'Add object to objects collected
						objectsCollected = objectsCollected + 1
						'Set status as collected
						objects(i, OBJECTSTATUS) = OBJECTCOLLECTED
						'Refresh collected objects in scoreboard
						showObjectsCollected()

					end if

				end if

				'-------------------------------------------------------------------------------------------------------------------
				'Door
				'-------------------------------------------------------------------------------------------------------------------
				if objectsstaticdata(objects(i, OBJECTID), OBJECTSTATICTYPE) = OBJECTDOOR then

			    	'We are next/touching the door
					if objects(i, OBJECTX) + sizeobjectx >= x and objects(i, OBJECTX) <= x + sizeherox _
						and objects(i, OBJECTY) + sizeobjecty >= y and objects(i, OBJECTY) <= y + sizeheroy then

						'If we have the key that is linked to this door
						if objects(objects(i, OBJECTLINKEDID), OBJECTSTATUS) = OBJECTCOLLECTED then

							'Removes door from screen
							removeObect(i)
							'Removes key from collected objects
							objectsCollected = objectsCollected - 1
							'Make door inactive (opened)
							objects(i, OBJECTSTATUS) = OBJECTINACTIVE
							'Set tiles as empty in the position of the door, so hero can trespass
							map(objects(i, OBJECTY)/TILESIZE, objects(i, OBJECTX)/TILESIZE, MAPTILE) = MAPTILEEMPTY 'Empty
							'Set key status to used
							objects(objects(i, OBJECTLINKEDID), OBJECTSTATUS) = OBJECTUSED
							'Refresh collected objects in scoreboard
							showObjectsCollected()
							'Reset jump, in case we were falling/jumping on the door
							'(there are screens that you can fall on it, open it, and then continue falling, and we don't want to die)
							resetJump()
						end if

					end if
	
				end if

				'-------------------------------------------------------------------------------------------------------------------
				'Final - Princess (End game)
				'-------------------------------------------------------------------------------------------------------------------
				if objectsstaticdata(objects(i, OBJECTID), OBJECTSTATICTYPE) = OBJECTFINAL then

			    	'We are next/touching the princess cage
					if objects(i, OBJECTX) + sizeobjectx >= x and objects(i, OBJECTX) <= x + sizeherox _
						and objects(i, OBJECTY) + sizeobjecty >= y and objects(i, OBJECTY) <= y + sizeheroy then

						'If we have the key that is linked to this cage door
						if objects(objects(i, OBJECTLINKEDID), OBJECTSTATUS) = OBJECTCOLLECTED then

							'Remove key from collected objects
							objectsCollected = objectsCollected - 1
							'Refresh collected objects in scoreboard
							showObjectsCollected()

							'Set tiles as empty in the position of the door, so hero can trespass
							map(objects(i, OBJECTY)/TILESIZE, objects(i, OBJECTX)/TILESIZE, MAPTILE) = MAPTILEEMPTY 'Empty

							'Sets the hero in front of princess without door
							sprites(HERO, SPRITEX) = objects(i, OBJECTX) - sizeherox + 1
							sprites(HERO, SPRITEY) = objects(i, OBJECTY)
							'TRUE - Print only the Hero, not the enemies, they are static
							printSprites(TRUE)

							'End game with success!
							gameend = TRUE

						end if

					end if
	
				end if

			end if
		end if

	next i

end sub

'-------------------------------------------------------------------------------------------------------------------
'Changes screen when we have arrived to a border and keep on moving.
'Updates screen vars and init vars to appear in the same position but opposite border
'when prionting next screen.
'-------------------------------------------------------------------------------------------------------------------
sub changeScreen(nextscreen as ubyte)

	dim changed as uByte = FALSE

	'New screen depends on the direction

	'-------------------------------------------------------------------------------------------------------------------
	'Moving right
	'-------------------------------------------------------------------------------------------------------------------
	if nextscreen = DIRRIGHT then

		'If not in the last screen right
		if currentscreenx < MAPWIDTH-1 then

			'Store last screen
			previousscreenx = currentscreenx
			previousscreeny = currentscreeny
			'Set the initial coordinates when entering in new screen
			initX = 0
			initY = sprites(HERO, SPRITEY)
			'Set current screen to screen to the right
			currentscreenx = currentscreenx + 1
			changed = TRUE

		end if

	'-------------------------------------------------------------------------------------------------------------------
	'Moving left
	'-------------------------------------------------------------------------------------------------------------------
	elseif nextscreen = DIRLEFT then

		'If not in the last screen left
		if currentscreenx > 0 then

			'Store last screen
			previousscreenx = currentscreenx
			previousscreeny = currentscreeny
			'Set the initial coordinates when entering in new screen
			initX = (MAPSCREENWIDTH * TILESIZE) - spritesstaticdata(sprites(HERO, SPRITEID), SPRITESTATICSIZEX)
			initY = sprites(HERO, SPRITEY)
			'Set current screen to screen to the left
			currentscreenx = currentscreenx - 1
			changed = TRUE

		end if

	'-------------------------------------------------------------------------------------------------------------------
	'Moving up
	'-------------------------------------------------------------------------------------------------------------------
	elseif nextscreen = DIRUP then

		'If not in the last screen up
		if currentscreeny > 0 then

			'Store last screen
			previousscreenx = currentscreenx
			previousscreeny = currentscreeny
			'Set the initial coordinates when entering in new screen
			initX = sprites(HERO, SPRITEX)
			initY = (MAPSCREENHEIGHT * TILESIZE) - spritesstaticdata(sprites(HERO, SPRITEID), SPRITESTATICSIZEY)
			'Set current screen to screen to the top
			currentscreeny = currentscreeny - 1
			changed = TRUE

		end if

	'-------------------------------------------------------------------------------------------------------------------
	'Moving down
	'-------------------------------------------------------------------------------------------------------------------
	elseif nextscreen = DIRDOWN then

		'If not in the last screen down
		if currentscreeny < MAPHEIGHT-1 then

			'Store last screen
			previousscreenx = currentscreenx
			previousscreeny = currentscreeny
			'Set the initial coordinates when entering in new screen
			initX = sprites(HERO, SPRITEX)
			initY = 0
			'Set current screen to screen to the bottom
			currentscreeny = currentscreeny + 1
			changed = TRUE

		end if

	end if

	'If we have changed screen
	if changed = TRUE then

		'Set initial vars, to start in new screen at same position but opposite border and jumping if that was the case
		initDirY = sprites(HERO, SPRITEDIRY)
		initDirX = sprites(HERO, SPRITEDIRX)
		initframe = sprites(HERO, SPRITEFRAME)
		initFrameJump = sprites(HERO, SPRITEFRAMEJUMP)
		initStatus = sprites(HERO, SPRITESTATUS)
		'Initialise new screen
		initScreen(currentscreeny, currentscreenx, FALSE)

	end if

end sub

'-------------------------------------------------------------------------------------------------------------------
'Print text centered on screen.
'We pass the Y position, the text, and the color of text.
'Depending on text length, we calculate the X position.
'-------------------------------------------------------------------------------------------------------------------
sub center(y as ubyte, t as string, color as uByte)
	
	'Print text centered in desired color
	print paper COLORBLACK; ink color; bright TRUE; at y + SCREENOFFSETY, (MAPSCREENWIDTH*TILESIZE - len(t))/2 + SCREENOFFSETX; t

end sub	

'-------------------------------------------------------------------------------------------------------------------
'Print message "GET READY" centered on a black screen.
'-------------------------------------------------------------------------------------------------------------------
sub getReady()

	'Clear screen
	border COLORBLACK: paper COLORBLACK: ink COLORWHITE: cls

	'Set charset to fontset
	Poke uInteger 23606, @fontset(0) - 256
	
	'Print message centered
	center((MAPSCREENHEIGHT*TILESIZE)/2, "53B @3/2G!", COLORWHITE) 'Get ready!

	'Pause does not work if you clicked a key during the game, they are still in memory
	'This is dirty, but it works to empty the keystrokes
	dim i as uInteger
	for i = 0 to 30000
	next i

	'Clear screen
	border COLORBLACK: paper COLORBLACK: ink COLORWHITE: cls

end sub

'-------------------------------------------------------------------------------------------------------------------
'Shows a text on bottom of the screen.
'Used to show var values on screen when runing the game, useful as we don't have debugging.
'It gives a warning when compiling if not used, but we ignore it.
'-------------------------------------------------------------------------------------------------------------------
sub debug(t as string)

	'Set charset to fontset
	Poke uInteger 23606, 15616 - 256

	'Print text at bottom
	print paper COLORBLACK; ink COLORWHITE; bright FALSE; at 23, 0; t

	'Pause does not work if you clicked a key during the game, they are still in memory
	'This is dirty, but it works to empty the keystrokes
	dim i as uInteger
	for i = 0 to 3000
	next i

end sub	

